<?php

namespace PM\PerfectMoneyPaymentGateway\Block\Widget;

/**
 * Abstract class
 */

use \Magento\Framework\View\Element\Template;


class Redirect extends Template
{
    /**
     * @var \PM\PerfectMoneyPaymentGateway\Model\PM
     */
    protected $Config;

    /**
     * @var \Magento\Checkout\Model\Session
     */
    protected $_checkoutSession;

    /**
     * @var \Magento\Customer\Model\Session
     */
    protected $_customerSession;

    /**
     * @var \Magento\Sales\Model\OrderFactory
     */
    protected $_orderFactory;

    /**
     * @var \Magento\Sales\Model\Order\Config
     */
    protected $_orderConfig;

    /**
     * @var \Magento\Framework\App\Http\Context
     */
    protected $httpContext;

    /**
     * @var string
     */
    protected $_template = 'html/pm_form.phtml';


    /**
     * @param Template\Context $context
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Customer\Model\Session $customerSession
     * @param \Magento\Sales\Model\OrderFactory $orderFactory
     * @param \Magento\Sales\Model\Order\Config $orderConfig
     * @param \Magento\Framework\App\Http\Context $httpContext
     * @param \PM\PerfectMoneyPaymentGateway\Model\PM $paymentConfig
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\View\Element\Template\Context $context,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Sales\Model\OrderFactory $orderFactory,
        \Magento\Sales\Model\Order\Config $orderConfig,
        \Magento\Framework\App\Http\Context $httpContext,
        \PM\PerfectMoneyPaymentGateway\Model\PM $paymentConfig,
        \Magento\Sales\Model\OrderRepository $orderRepository,
        array $data = []
    )
    {
        parent::__construct($context, $data);
        $this->_checkoutSession = $checkoutSession;
        $this->_customerSession = $customerSession;
        $this->_orderFactory = $orderFactory;
        $this->_orderConfig = $orderConfig;
        $this->_isScopePrivate = true;
        $this->httpContext = $httpContext;
        $this->Config = $paymentConfig;
        $this->_orderRepository = $orderRepository;
    }


    public function getGateUrl()
    {
        return $this->Config->getGateUrl();
    }


    public function getAmount()
    {
        $orderId = $this->_checkoutSession->getLastOrderId();

        if ($orderId) {
            $incrementId = $this->_checkoutSession->getLastRealOrderId();
            return $this->Config->getAmount($incrementId);
        }

        return ['error' => 'No data'];
    }

    public function getPostData($data = [])
    {

        $orderId = $this->_checkoutSession->getLastOrderId();

        if ($orderId or isset($data['order'])) {
            $incrementId = $this->_checkoutSession->getLastRealOrderId();
            if (!$incrementId) {
                $order = $this->_orderRepository->get($data['order']);
                if ($order) {
                    if ($order->getStatus() == 'pending' and $order->getState() == 'new') {
                        $incrementId = $order->getIncrementId();
                    }
                }
            }
            if (!$incrementId){
                return ['error' => 'No data'];
            }
            return $this->Config->getPostData($incrementId);
        }

        return ['error' => 'No data'];
    }

    public function getPayUrl()
    {
        $baseUrl = $this->getUrl("pm/url");
        return "{$baseUrl}success";
    }
}
