<?php

namespace PM\PerfectMoneyPaymentGateway\Controller\Url;

use Magento\Framework\App\Action\Action;
use Magento\Sales\Model\Order;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\App\CsrfAwareActionInterface;
use Magento\Framework\App\Request\InvalidRequestException;
use Magento\Framework\App\RequestInterface;

class Submit extends Action implements CsrfAwareActionInterface
{

    public $resultPageFactory;

    public $pm;

    protected $_checkoutSession;

    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory,
        \PM\PerfectMoneyPaymentGateway\Block\Widget\Redirect $pm_form
    )
    {
        $this->resultPageFactory = $resultPageFactory;
        $this->_checkoutSession = $checkoutSession;
        $this->pm = $pm_form;
        $this->_isScopePrivate = true;
        parent::__construct($context);
    }

    public function execute()
    {
        $data = $this->getRequest()->getParams();
        $post_data = $this->pm->getPostData($data);
        
        if (isset($post_data['error'])){
            $message = __('Payment failed. No items in cart. Please try again.');
            $this->messageManager->addError($message);
            return $this->resultRedirectFactory->create()->setPath('checkout/cart');
        }
        
        $page = $this->resultPageFactory->create();
        return $page;
    }

    public function createCsrfValidationException(
        RequestInterface $request
    ): InvalidRequestException
    {
        return null;
    }

    public function validateForCsrf(RequestInterface $request): bool
    {
        return true;
    }
    
    public function restoreCart()
    {
        $lastQuoteId = $this->_checkoutSession->getLastQuoteId();
        if ($quote = $this->_objectManager->get('Magento\Quote\Model\Quote')->loadByIdWithoutStore($lastQuoteId)) {
            $quote->setIsActive(true)
                ->setReservedOrderId(null)
                ->save();
            $this->_checkoutSession->setQuoteId($lastQuoteId);
        }
        $message = __('Payment failed. Please try again.');
        $this->messageManager->addError($message);
    }
    
}
