<?php

namespace PM\PerfectMoneyPaymentGateway\Model;

use Magento\Quote\Api\Data\CartInterface;
use Magento\Sales\Model\Order\Payment\Transaction;
use Magento\Sales\Model\Order;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Encryption\EncryptorInterface;

/**
 * Class PM
 * @package PM\PerfectMoneyPaymentGateway\Model
 */
class PM extends \Magento\Payment\Model\Method\AbstractMethod
{
    /**
     * @var bool
     */
    protected $_isInitializeNeeded = true;
    /**
     * @var bool
     */
    protected $_isGateway = false;
    /**
     * Payment code
     *
     * @var string
     */
    protected $_code = 'pm_gateway';

    /**
     * Availability option
     *
     * @var bool
     */
    protected $_isOffline = false;
    /**
     * Sidebar payment info block
     *
     * @var string
     */
    protected $_infoBlockType = 'Magento\Payment\Block\Info\Instructions';

    protected $_gateUrl = "https://perfectmoney.com/api/step1.asp";

    protected $_encryptor;

    protected $orderFactory;

    protected $urlBuilder;

    protected $_transactionBuilder;

    protected $_logger;

    protected $_canUseCheckout = true;

    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory,
        \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory,
        \Magento\Payment\Helper\Data $paymentData,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Payment\Model\Method\Logger $logger,
        \Magento\Framework\Module\ModuleListInterface $moduleList,
        \Magento\Framework\Stdlib\DateTime\TimezoneInterface $localeDate,
        \Magento\Framework\Encryption\EncryptorInterface $encryptor,
        \Magento\Framework\UrlInterface $urlBuilder,
        \Magento\Sales\Model\Order\Payment\Transaction\BuilderInterface $builderInterface,
        \Magento\Sales\Model\OrderFactory $orderFactory,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    )
    {
        $this->orderFactory = $orderFactory;
        $this->urlBuilder = $urlBuilder;
        $this->_transactionBuilder = $builderInterface;
        $this->_encryptor = $encryptor;
        parent::__construct($context,
            $registry,
            $extensionFactory,
            $customAttributeFactory,
            $paymentData,
            $scopeConfig,
            $logger,
            $resource,
            $resourceCollection,
            $data);
        $writer = new \Zend\Log\Writer\Stream(BP . '/var/log/pm.log');
        $this->_logger = new \Zend\Log\Logger();
        $this->_logger->addWriter($writer);
        $this->_gateUrl = 'https://perfectmoney.com/api/step1.asp';
    }

    protected function getOrder($orderId)
    {
        return $this->orderFactory->create()->loadByIncrementId($orderId);
    }

    public function getAmount($order)
    {
        return $order->getGrandTotal();
    }

    public function getSignature($data, $password, $encoded = true)
    {
        $data = array_filter($data, function ($var) {
            return $var !== '' && $var !== null;
        });
        ksort($data);
        $str = $password;
        foreach ($data as $k => $v) {
            $str .= '|' . $v;
        }
        if ($encoded) {
            return sha1($str);
        } else {
            return $str;
        }
    }

    public function getCustomerId($orderId)
    {
        return $this->getOrder($orderId)->getCustomerId();
    }

    public function getCurrencyCode($order)
    {
        return $order->getBaseCurrencyCode();
    }

    public function getMerchantDataString($order)
    {
        $addData = $order->getBillingAddress()->getData();
        if (!$addData){
            $addData = $order->getShippigAddress()->getData();
        }
        if ($addData){
            $addInfo = [
                'Fullname' => $addData['firstname'] . ' ' . $addData['middlename'] . ' ' . $addData['lastname']
            ];
            return $addInfo;
        }
        return null;
    }

    public function getReservDataString($order)
    {
        $addData = $order->getBillingAddress()->getData();
        if (!$addData){
            $addData = $order->getShippigAddress()->getData();
        }

        $skuString = '';
        try {
            $orderItems = $order->getAllVisibleItems();
            $countItems = count($orderItems);
            $i = 0;
            foreach ($orderItems as $key => $orderItem) {
                $sku = $orderItem->getData()['sku'];
                if ($countItems > 1) {
                    $skuString .= ++$i === $countItems ? $sku . '' : $sku . ', ';
                } else {
                    $skuString .= $sku;
                }
            }
        } catch (Exception $e) {
            $skuString = "No sku";
            $this->_logger->debug("Cant get products sku");
        }

        $addInfo = [
            'customer_zip' => isset($addData['postcode']) ? $addData['postcode'] : '',
            'customer_name' => $addData['firstname'] . ' ' . $addData['middlename'] . ' ' . $addData['lastname'],
            'customer_address' => isset($addData['street']) ? $addData['street'] : '',
            'customer_state' => isset($addData['region_id']) ? $addData['region_id'] : '',
            'customer_country' => isset($addData['country_id']) ? $addData['country_id'] : '',
            'phonemobile' => isset($addData['telephone']) ? $addData['telephone'] : '',
            'account' => isset($addData['email']) ? $addData['email'] : '',
            'products_sku' => isset($skuString) ? $skuString : ''
        ];

        try {
            $addInfo['Shipping total'] = number_format($order->getShippingAmount(), 2, '.', '');
        } catch (Exception $e) {
            $this->_logger->debug("Can't get products shipping price");
        }

        return $addInfo;
    }

    protected function isCarrierAllowed($shippingMethod)
    {
        $allowedConfig = $this->getConfigData('allowed_carrier');

        if ($allowedConfig == '' || !$allowedConfig) {
            return true;
        }

        $allow = explode(',', $allowedConfig);
        foreach ($allow as $v) {
            if (preg_match("/{$v}/i", $shippingMethod)) {
                return true;
            }
        }

        return strpos($allowedConfig, $shippingMethod) !== false;
    }

    public function isAvailable(\Magento\Quote\Api\Data\CartInterface $quote = null)
    {
        if ($quote === null) {
            return false;
        }
        return parent::isAvailable($quote) && $this->isCarrierAllowed(
                $quote->getShippingAddress()->getShippingMethod()
            );
    }

    public function getGateUrl()
    {
        return $this->_gateUrl;
    }

    public function getDataIntegrityCode()
    {
        return $this->_encryptor->decrypt($this->getConfigData('PM_SECRET_KEY'));
    }

    public function getPostData($orderId)
    {
        $order = $this->getOrder($orderId);
        
        if (!$order){
            return ['error' => 'No data'];
        }

        $merchant_data = $this->getMerchantDataString($order);
        $reservation_data = $this->getReservDataString($order);
        $email = $order->getCustomerEmail();
        $postData = array(
            'order_id' => $orderId,
            'merchant_account' => $this->getConfigData("merchant_account"),
            'amount' => round(number_format($this->getAmount($order), 2, '.', ''), 2),
            'order_desc' => __("Pay for order #") . $orderId,
            'sender_email' => $email,
            'product_id' => 'PM',
            'server_callback_url' => $this->urlBuilder->getUrl('pm/url/status'),
            'response_url' => $this->urlBuilder->getUrl('pm/url/response'),
            'currency' => $this->getCurrencyCode($order)
        );
        if (!empty($merchant_data)) {
            $postData['merchant_data'] = json_encode(array($merchant_data));
            $reservation_data['order_id'] = $orderId;
            $reservation_data['order_total'] = number_format($this->getAmount($order), 2, '.', '');
            $postData['reservation_data'] = base64_encode(json_encode($reservation_data));
        }

        if ($this->getConfigData("invoice_before_fraud_review")) {
            $postData['preauth'] = "Y";
        }

        $sign = $this->getSignature($postData, $this->getDataIntegrityCode());
        $postData['signature'] = $sign;
        
        $postData['store_name']=$storeName = \Magento\Framework\App\ObjectManager::getInstance()
            ->get(\Magento\Store\Model\StoreManagerInterface::class)
            ->getStore()
            ->getName();
        
        return $postData;
    }

    protected function _processOrder(Order $order, $post_data)
    {
        
        $this->createTransaction($order, $post_data);
        
        $order_status = $this->getConfigData("order_status");
        
        if ($order_status == 'pending') {
            //Preevent incorrect status
            $order_status = 'processing';
        }
        
        $order
            ->setState($order_status)
            ->setStatus('processing')
            ->save();
    }
    
    public function processResponse($post_data)
    {
        
        $orderId=(int)$post_data['PAYMENT_ID'];
        
        $order = $this->getOrder($orderId);
        
        if(is_object($order)){
            
            $string=
                $post_data['PAYMENT_ID'].':'.$post_data['PAYEE_ACCOUNT'].':'.
                $post_data['PAYMENT_AMOUNT'].':'.$post_data['PAYMENT_UNITS'].':'.
                $post_data['PAYMENT_BATCH_NUM'].':'.$post_data['PAYER_ACCOUNT'].':'.
                strtoupper(md5($this->_encryptor->decrypt($this->getConfigData("merchant_phrase")))).':'.
                $post_data['TIMESTAMPGMT'];
            
            $hash=strtoupper(md5($string));
            
            if($hash===$post_data['V2_HASH']){
                
                $order_amount=round(number_format($order->getGrandTotal(), 2, '.', ''), 2);
                
                if($post_data['PAYMENT_AMOUNT']==$order_amount && 
                        $post_data['PAYEE_ACCOUNT']==$this->getConfigData("merchant_account") && 
                        $post_data['PAYMENT_UNITS']=='USD'){
                    
                    $order->addStatusHistoryComment("Perfect Money payment has been made. Payment batch No.: " . $post_data['PAYMENT_BATCH_NUM']);

                    $this->_processOrder($order, $post_data);
                    
                }

             }
            
        }
        
    }

    public function createTransaction($order = null, $paymentData = array())
    {
        try {
            //get payment object from order object
            $payment = $order->getPayment();

            $payment->setLastTransId($paymentData['PAYMENT_BATCH_NUM']);
            $payment->setTransactionId($paymentData['PAYMENT_BATCH_NUM']);
            $payment->setAdditionalInformation(
                [\Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS => (array)$paymentData]
            );
            $formatedPrice = $order->getBaseCurrency()->formatTxt(
                $order->getGrandTotal()
            );

            //$message = __('The authorized amount is %1.', $formatedPrice);
            //get the object of builder class
            $trans = $this->_transactionBuilder;
            $transaction = $trans->setPayment($payment)
                ->setOrder($order)
                ->setTransactionId($paymentData['PAYMENT_BATCH_NUM'])
                ->setAdditionalInformation(
                    [\Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS => (array)$paymentData]
                )
                ->setFailSafe(true)
                //build method creates the transaction and returns the object
                ->build(\Magento\Sales\Model\Order\Payment\Transaction::TYPE_ORDER);

            $payment->addTransactionCommentsToOrder(
                $transaction,
                $message
            );
            $payment->setParentTransactionId(null);
            $payment->save();
            $order->save();

            return $transaction->save()->getTransactionId();

        } catch (\Exception $e) {
            $this->_logger->debug("_processOrder exception", $e->getTrace());
            return false;
        }
    }

    public function getConfigData($field, $storeId = null)
    {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $storeManager = $objectManager->get('\Magento\Store\Model\StoreManagerInterface');

        if (null === $storeId) {
            $storeId = $storeManager->getStore()->getStoreId();
        }
        $path = 'payment/' . $this->_code . '/' . $field;
        return $this->_scopeConfig->getValue($path, \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $storeId);
    }
    
}
